@@ -8,21 +8,32 @@ oauthlib.oauth2.draft_25
 This module is an implementation of various logic needed
 for signing and checking OAuth 2.0 draft 25 requests.
 """
+import logging
+
 from oauthlib.common import Request
-from oauthlib.oauth2.draft25 import errors
-from .tokens import prepare_bearer_uri, prepare_bearer_headers
-from .tokens import prepare_bearer_body, prepare_mac_header
-from .tokens import BearerToken
+from oauthlib.oauth2.draft25 import tokens, grant_types
 from .parameters import prepare_grant_uri, prepare_token_request
 from .parameters import parse_authorization_code_response
 from .parameters import parse_implicit_response, parse_token_response
-from .utils import params_from_uri
 
 
 AUTH_HEADER = 'auth_header'
 URI_QUERY = 'query'
 BODY = 'body'
 
+log = logging.getLogger('oauthlib')
+
+# Add a NullHandler to prevent warnings for users who don't wish
+# to configure logging.
+try:
+    log.addHandler(logging.NullHandler())
+# NullHandler gracefully backported to 2.6
+except AttributeError:
+    class NullHandler(logging.Handler):
+        def emit(self, record):
+            pass
+    log.addHandler(NullHandler())
+
 
 class Client(object):
     """Base OAuth2 client responsible for access tokens.
@@ -134,13 +145,13 @@ class Client(object):
             headers=None, token_placement=None):
         """Add a bearer token to the request uri, body or authorization header."""
         if token_placement == AUTH_HEADER:
-            headers = prepare_bearer_headers(self.access_token, headers)
+            headers = tokens.prepare_bearer_headers(self.access_token, headers)
 
         elif token_placement == URI_QUERY:
-            uri = prepare_bearer_uri(self.access_token, uri)
+            uri = tokens.prepare_bearer_uri(self.access_token, uri)
 
         elif token_placement == BODY:
-            body = prepare_bearer_body(self.access_token, body)
+            body = tokens.prepare_bearer_body(self.access_token, body)
 
         else:
             raise ValueError("Invalid token placement.")
@@ -152,9 +163,9 @@ class Client(object):
 
         Warning: MAC token support is experimental as the spec is not yet stable.
         """
-        headers = prepare_mac_header(self.access_token, uri, self.mac_key, http_method,
-                        headers=headers, body=body, ext=ext,
-                        hash_algorithm=self.mac_algorithm, **kwargs)
+        headers = tokens.prepare_mac_header(self.access_token, uri,
+                self.mac_key, http_method, headers=headers, body=body, ext=ext,
+                hash_algorithm=self.mac_algorithm, **kwargs)
         return uri, headers, body
 
     def _populate_attributes(self, response):
@@ -567,109 +578,116 @@ class AuthorizationEndpoint(object):
     MUST NOT be included more than once.
     """
 
-    def __init__(self, default_token=None, response_types=None):
-        self._response_types = response_types or {}
-        self._default_token = default_token or BearerToken()
+    def __init__(self, default_response_type, default_token_type,
+            response_types):
+        self._response_types = response_types
+        self._default_response_type = default_response_type
+        self._default_token_type = default_token_type
 
     @property
     def response_types(self):
         return self._response_types
 
     @property
-    def default_token(self):
-        return self._default_token
+    def default_response_type(self):
+        return self._default_response_type
+
+    @property
+    def default_response_type_handler(self):
+        return self.response_types.get(self.default_response_type)
+
+    @property
+    def default_token_type(self):
+        return self._default_token_type
 
     def create_authorization_response(self, uri, http_method='GET', body=None,
+            headers=None, scopes=None, credentials=None):
+        """Extract response_type and route to the designated handler."""
+        request = Request(uri, http_method=http_method, body=body, headers=headers)
+        request.authorized_scopes = scopes
+        for k, v in (credentials or {}).items():
+            setattr(request, k, v)
+        response_type_handler = self.response_types.get(
+                request.response_type, self.default_response_type_handler)
+        log.debug('Dispatching response_type %s request to %r.',
+                  request.response_type, response_type_handler)
+        return response_type_handler.create_authorization_response(
+                        request, self.default_token_type)
+
+    def validate_authorization_request(self, uri, http_method='GET', body=None,
             headers=None):
         """Extract response_type and route to the designated handler."""
         request = Request(uri, http_method=http_method, body=body, headers=headers)
-        query_params = params_from_uri(self.request.uri)
-        body_params = self.request.decoded_body
-
-        # Prioritize response_type defined as query param over those in body.
-        # Chosen because the two core grant types utilizing the response type
-        # parameter both supply it in the uri. However it is not specified
-        # explicitely in RFC 6748.
-        if 'response_type' in query_params:
-            request.response_type = query_params.get('response_type')
-        elif 'response_type' in body_params:
-            request.response_type = body_params.get('response_type')
-        else:
-            raise errors.InvalidRequestError(
-                description='The response_type parameter is missing.')
-
-        if not request.response_type in self.response_types:
-            raise errors.UnsupportedResponseTypeError(
-                description='Invalid response type')
-
-        return self.response_types.get(
-                request.response_type).create_authorization_response(
-                        request, self.default_token)
+        response_type_handler = self.response_types.get(
+                request.response_type, self.default_response_type_handler)
+        return response_type_handler.validate_authorization_request(request)
 
 
 class TokenEndpoint(object):
 
-    def __init__(self, default_token=None, grant_types=None):
-        self._grant_types = grant_types or {}
-        self._default_token = default_token or BearerToken()
+    def __init__(self, default_grant_type, default_token_type, grant_types):
+        self._grant_types = grant_types
+        self._default_token_type = default_token_type
+        self._default_grant_type = default_grant_type
 
     @property
     def grant_types(self):
         return self._grant_types
 
     @property
-    def default_token(self):
-        return self._default_token
+    def default_grant_type(self):
+        return self._default_grant_type
 
-    def create_token_response(self, uri, http_method='GET', body=None, headers=None):
+    @property
+    def default_grant_type_handler(self):
+        return self.grant_types.get(self.default_grant_type)
+
+    @property
+    def default_token_type(self):
+        return self._default_token_type
+
+    def create_token_response(self, uri, http_method='GET', body=None,
+            headers=None, credentials=None):
         """Extract grant_type and route to the designated handler."""
         request = Request(uri, http_method=http_method, body=body, headers=headers)
-        query_params = params_from_uri(self.request.uri)
-        body_params = self.request.decoded_body
-
-        # Prioritize grant_type defined as body param over those in uri.
-        # Chosen because all three core grant types supply this parameter
-        # in the body. However it is not specified explicitely in RFC 6748.
-        if 'grant_type' in body_params:
-            request.grant_type = query_params.get('grant_type')
-        elif 'grant_type' in query_params:
-            request.grant_type = body_params.get('grant_type')
-        else:
-            raise errors.InvalidRequestError(
-                description='The grant_type parameter is missing.')
-
-        if not request.grant_type in self.grant_types:
-            raise errors.UnsupportedGrantTypeError(
-                description='Invalid response type')
-
-        return self.grant_types.get(
-                request.grant_type).create_token_response(
-                        request, self.default_token)
+        request.extra_credentials = credentials
+        grant_type_handler = self.grant_types.get(request.grant_type,
+                self.default_grant_type_handler)
+        log.debug('Dispatching grant_type %s request to %r.',
+                  request.grant_type, grant_type_handler)
+        return grant_type_handler.create_token_response(
+                request, self.default_token_type)
 
 
 class ResourceEndpoint(object):
 
-    def __init__(self, tokens=None):
-        self._tokens = tokens or {'Bearer': BearerToken()}
+    def __init__(self, default_token, token_types):
+        self._tokens = token_types
+        self._default_token = default_token
+
+    @property
+    def default_token(self):
+        return self._default_token
+
+    @property
+    def default_token_type_handler(self):
+        return self.tokens.get(self.default_token)
 
     @property
     def tokens(self):
         return self._tokens
 
-    def verify_request(self, uri, http_method='GET', body=None, headers=None):
+    def verify_request(self, uri, http_method='GET', body=None, headers=None,
+            scopes=None):
         """Validate client, code etc, return body + headers"""
         request = Request(uri, http_method, body, headers)
         request.token_type = self.find_token_type(request)
-
-        # TODO(ib-lundgren): How to return errors is not strictly defined and
-        # should allow for customization.
-        if not request.token_type:
-            raise ValueError('Could not determine the token type.')
-
-        if not request.token_type in self.tokens:
-            raise ValueError('Unsupported token type.')
-
-        return self.tokens.get(request.token_type).validate_request(request)
+        request.scopes = scopes
+        token_type_handler = self.tokens.get(request.token_type,
+                self.default_token_type_handler)
+        log.debug('Dispatching token_type %s request to %r.',
+                  request.token_type, token_type_handler)
+        return token_type_handler.validate_request(request), request
 
     def find_token_type(self, request):
         """Token type identification.
@@ -679,9 +697,95 @@ class ResourceEndpoint(object):
         the most likely token type (if any) by asking each known token type
         to give an estimation based on the request.
         """
-        estimates = sorted((t.estimate_type(request) for t in self.tokens))
-        return estimates[0] if len(estimates) else None
+        estimates = sorted(((t.estimate_type(request), n) for n, t in self.tokens.items()))
+        return estimates[0][1] if len(estimates) else None
 
 
 class Server(AuthorizationEndpoint, TokenEndpoint, ResourceEndpoint):
-    pass
+    """An all-in-one endpoint featuring all four major grant types."""
+
+    def __init__(self, request_validator, *args, **kwargs):
+        auth_grant = grant_types.AuthorizationCodeGrant(request_validator)
+        implicit_grant = grant_types.ImplicitGrant(request_validator)
+        password_grant = grant_types.ResourceOwnerPasswordCredentialsGrant(request_validator)
+        credentials_grant = grant_types.ClientCredentialsGrant(request_validator)
+        refresh_grant = grant_types.RefreshTokenGrant(request_validator)
+        bearer = tokens.BearerToken(request_validator)
+        AuthorizationEndpoint.__init__(self, default_response_type='code',
+                response_types={
+                    'code': auth_grant,
+                    'token': implicit_grant,
+                },
+                default_token_type=bearer)
+        TokenEndpoint.__init__(self, default_grant_type='authorization_code',
+                grant_types={
+                    'authorization_code': auth_grant,
+                    'password': password_grant,
+                    'client_credentials': credentials_grant,
+                    'refresh_token': refresh_grant,
+                },
+                default_token_type=bearer)
+        ResourceEndpoint.__init__(self, default_token='Bearer',
+                token_types={'Bearer': bearer})
+
+
+class WebApplicationServer(AuthorizationEndpoint, TokenEndpoint, ResourceEndpoint):
+    """An all-in-one endpoint featuring Authorization code grant and Bearer tokens."""
+
+    def __init__(self, request_validator, *args, **kwargs):
+        auth_grant = grant_types.AuthorizationCodeGrant(request_validator)
+        refresh_grant = grant_types.RefreshTokenGrant(request_validator)
+        bearer = tokens.BearerToken(request_validator)
+        AuthorizationEndpoint.__init__(self, default_response_type='code',
+                response_types={'code': auth_grant})
+        TokenEndpoint.__init__(self, default_grant_type='authorization_code',
+                grant_types={
+                    'authorization_code': auth_grant,
+                    'refresh_token': refresh_grant,
+                },
+                default_token_type=bearer)
+        ResourceEndpoint.__init__(self, default_token='Bearer',
+                token_types={'Bearer': bearer})
+
+
+class MobileApplicationServer(AuthorizationEndpoint, ResourceEndpoint):
+    """An all-in-one endpoint featuring Implicit code grant and Bearer tokens."""
+
+    def __init__(self, request_validator, *args, **kwargs):
+        implicit_grant = grant_types.ImplicitGrant(request_validator)
+        bearer = tokens.BearerToken(request_validator)
+        AuthorizationEndpoint.__init__(self, default_response_type='token',
+                response_types={'token': implicit_grant},
+                default_token_type=bearer)
+        ResourceEndpoint.__init__(self, default_token='Bearer',
+                token_types={'Bearer': bearer})
+
+
+class LegacyApplicationServer(TokenEndpoint, ResourceEndpoint):
+    """An all-in-one endpoint featuring Authorization code grant and Bearer tokens."""
+
+    def __init__(self, request_validator, *args, **kwargs):
+        password_grant = grant_types.ResourceOwnerPasswordCredentialsGrant(request_validator)
+        refresh_grant = grant_types.RefreshTokenGrant(request_validator)
+        bearer = tokens.BearerToken(request_validator)
+        TokenEndpoint.__init__(self, default_grant_type='password',
+                grant_types={
+                    'password': password_grant,
+                    'refresh_token': refresh_grant,
+                },
+                default_token_type=bearer)
+        ResourceEndpoint.__init__(self, default_token='Bearer',
+                token_types={'Bearer': bearer})
+
+
+class BackendApplicationServer(TokenEndpoint, ResourceEndpoint):
+    """An all-in-one endpoint featuring Authorization code grant and Bearer tokens."""
+
+    def __init__(self, request_validator, *args, **kwargs):
+        credentials_grant = grant_types.ClientCredentialsGrant(request_validator)
+        bearer = tokens.BearerToken(request_validator)
+        TokenEndpoint.__init__(self, default_grant_type='client_credentials',
+                grant_types={'client_credentials': credentials_grant},
+                default_token_type=bearer)
+        ResourceEndpoint.__init__(self, default_token='Bearer',
+                token_types={'Bearer': bearer})
